/***************************************************************************
 *   Copyright (C) 2009 by QEVEN Team                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 **************************************************************************/

#ifndef MAIN_WINDOW_H
#define MAIN_WINDOW_H

#include <QMainWindow>
#include <QDomDocument>
#include <QSystemTrayIcon>
#include "ui_Main_gui.h"
#include "options_gui.h"
#include "converter_gui.h"
#include "project_gui.h"
#include "video_manager.h"

class AboutUs;
using namespace video_manager;



class Main_Window : public QMainWindow,
private Ui::Main_Form
{
   Q_OBJECT

public:
   Main_Window(QWidget* parent = 0, Qt::WFlags fl = 0 );
   ~Main_Window();

   void Update_Space_Bar();
   void Update_List();
   void Enable_Buttons();
   void Disable_Buttons();
   void SetProgramDir(QString MEncoder, QString MPlayer);
   void createTrayIcon();
   void createMenu();
   void createConnection();



public slots:
   void Add_Video(); //Adiciona un video
   void Remove_Video(); //Elimina un video
   void About(); //Acerca del proyecto
   void Disk_Changed(); //Cuando se cambia entre CD y DVD
   void Project_Selected( const e_video_format& project ); //Tipo de Proyecto
   void Start_Conversion(); //Comenzar a convertir
   void Show_Options(); //Mostrar las opciones del video seleccionado
   void Read_Options_Changed( const int& item_selected,
                              const int& vbitrate,
                              const int& abitrate,
                              const QString& vaspect,
                              const bool& has_subtitles,
                              const QString& sub_path,
                              const QString& sub_font,
                              const int& sub_scale,
                              const int& resolution,
                              const int& width,
                              const int& height); //Lee los cambios realizados en las opciones
   void ApplyAll_Options_Changed(const int& vbitrate,
                                 const int& abitrate,
                                 const QString& vaspect,
                                 const QString& sub_font,
                                 const int& sub_scale,
                                 const int& resolution,
                                 const int& width,
                                 const int& height);
   void Exit();
   void Save();
   void Save( QString );
   bool maybeSave();
   void SaveForce();
   void Load();
   void Load(QString);
   void Reset();
   void New();
   void Show_Wizard();
   void ImportFont();
   void add_Font(QString);
   void Show_Main_Window();
   void ChangeLanguage();

   void iconActivated(QSystemTrayIcon::ActivationReason reason);

protected:
   void parseVideoElement( const QDomElement &element );
   void dropEvent( QDropEvent* );
   void dragEnterEvent( QDragEnterEvent* );
   void closeEvent( QCloseEvent *event);
   void keyReleaseEvent(QKeyEvent * event);

private:
   void readSettings();
   void writeSettings();
   QStringList findQmFiles();
   QString languageName(const QString &qmFile);

   Project_GUI* wizard_gui;
   Options_GUI* options_gui;
   Converter_GUI* converter_gui;
   Video_Manager* m_video_list;
   AboutUs* about;
   QString lastProjectDir;
   QString projectFileName;
   bool modified;
   bool flagClose;
   int visible_Window; //para controlar que ventana esta visible.
   // y visualizarla cuando le doy click al systray
   bool firstTrayMsg;

   QSystemTrayIcon *trayIcon;
   QMenu *trayIconMenu;
   QActionGroup* Language;
//   QAction* LEnglish;
//   QAction* LSpanish;


   QString realLang;
  // QList<QAction*> Lenguas;
   int cantLeng;


   QString MEncoder;
   QString MPlayer;
};

#endif

