/***************************************************************************
 *   Copyright (C) 2009 by QEVEN Team                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 **************************************************************************/

#include <QSettings>
#include <QDesktopWidget>
#include <QFileDialog>
#include "options_gui.h"



Options_GUI::Options_GUI( QWidget* parent, Qt::WFlags fl ) :
      QWidget( parent, fl ),
      Ui::Options_Form()
{
   setupUi( this );

#ifdef Q_OS_LINUX
   if(qApp->applicationDirPath() == "/usr/bin")
      font_dir = "/usr/share/qeven/fonts/";
   else
      font_dir = "./Fonts/";
#endif

   connect( acceptButton, SIGNAL( clicked() ),
            this, SLOT( Accept_Options() ) );
   connect( cancelButton, SIGNAL( clicked() ),
            this, SLOT( close() ) );
   connect( pbApplyAll, SIGNAL( clicked()),
            this,SLOT( ApplyAll_Options()));
   connect( subtitlesCheckBox, SIGNAL( stateChanged( int ) ),
            this, SLOT( Check_Subtitles() ) );
   connect( browseButton, SIGNAL( clicked() ),
            this, SLOT( Add_Subtitles() ) );
   connect(pb_FontImport,SIGNAL(clicked()),
           this,SLOT(ImportFont()));

   cantFUsr = 0;
   cantFApp = 0;
   cantFSys = 0;
   fuentes();
   readSettings();
   fileDir = QDir::homePath();

   CorrectFont(fontComboBox->currentIndex());
}

Options_GUI::~Options_GUI()
{
}

//---- SLOTS ------------------
/**
 * SLOT. Se ejecuta cuando se Aceptan las Opciones.
 * Emite una signal con las opciones del video.
 */
void Options_GUI::Accept_Options()
{
   QString vaspect;

   switch( aspectComboBox->currentIndex() )
   {
        case 0 :
      vaspect = ":autoaspect";
      break;
        case 1 :
      vaspect = ":aspect=16/9";
      break;
        case 2 :
      vaspect = ":aspect=4/3";
      break;
   }

   if( subtitlesCheckBox->isChecked() )
   {
      QString fontp = "";

#ifdef Q_OS_LINUX
      if(fontComboBox->currentIndex() < cantFUsr + 1)
         fontp = QDir::homePath() + "/.QEVEN/Fonts/" + fontComboBox->currentText();
      else
         fontp = font_dir + fontComboBox->currentText();
#endif
#ifdef Q_OS_WIN32
      if(fontComboBox->currentIndex() < cantFUsr + 1)
      {
         QSettings ini(QSettings::IniFormat,QSettings::UserScope,
                       QCoreApplication::organizationName(),
                       QCoreApplication::applicationName());
         QDir userdir = QDir(QFileInfo(ini.fileName()).absolutePath()+"/QEVEN/Fonts");
         fontp = userdir.path() + "/" + fontComboBox->currentText();
      }
      else
      {
         fontp = QDir(".\\Fonts\\").path() + fontComboBox->currentText();
      }

#endif


      if(fontComboBox->currentIndex() < cantFUsr + 1)
         fontp = QDir::homePath() + "/.QEVEN/Fonts/" + fontComboBox->currentText();
      else
         fontp = font_dir + fontComboBox->currentText();

      emit signal_options_changed( m_item_selected,
                                   videobitrateSpin->value(),
                                   audiobitrateSpin->value(),
                                   vaspect,
                                   subtitlesCheckBox->isChecked(),
                                   subsEdit->text(),
                                   fontp,
                                   scaleSpinBox->value(),
                                   comboBox->currentIndex(),
                                   spwidth->value(),
                                   spheight->value());
   }
   else
   {
      emit signal_options_changed( m_item_selected,
                                   videobitrateSpin->value(),
                                   audiobitrateSpin->value(),
                                   vaspect,
                                   subtitlesCheckBox->isChecked(),
                                   "",
                                   "",
                                   scaleSpinBox->value(),
                                   comboBox->currentIndex(),
                                   spwidth->value(),
                                   spheight->value());
   }

   this->close();
}

/**
  * [SLOT] Se ejecuta cuando se Aplican a Todos las Opciones
  * Emite una signal con las nuevas opciones para todos los vides
  */
void Options_GUI::ApplyAll_Options()
{
   QString vaspect;

   switch( aspectComboBox->currentIndex() )
   {
        case 0 :
      vaspect = ":autoaspect";
      break;
        case 1 :
      vaspect = ":aspect=16/9";
      break;
        case 2 :
      vaspect = ":aspect=4/3";
      break;
   }

   QString fontp = "";

#ifdef Q_OS_LINUX
   if(fontComboBox->currentIndex() < cantFUsr + 1)
      fontp = QDir::homePath() + "/.QEVEN/Fonts/" + fontComboBox->currentText();
   else
      fontp = font_dir + fontComboBox->currentText();
#endif
#ifdef Q_OS_WIN32
   if(fontComboBox->currentIndex() < cantFUsr + 1)
   {
      QSettings ini(QSettings::IniFormat,QSettings::UserScope,
                    QCoreApplication::organizationName(),
                    QCoreApplication::applicationName());
      QDir userdir = QDir(QFileInfo(ini.fileName()).absolutePath()+"/QEVEN/Fonts");
      fontp = userdir.path() + "/" + fontComboBox->currentText();
   }
   else
   {
      fontp = QDir(".\\Fonts\\").path() + fontComboBox->currentText();
   }

#endif
   emit signal_options_applyall(videobitrateSpin->value(),
                                audiobitrateSpin->value(),
                                vaspect,
                                fontp,
                                scaleSpinBox->value(),
                                comboBox->currentIndex(),
                                spwidth->value(),
                                spheight->value());

   this->close();
}


/**
 * SLOT. habilita o deshabilita el Groupbox
 * con las opciones de los subtitulos.
 */
void Options_GUI::Check_Subtitles()
{
   if( subtitlesCheckBox->isChecked() )
   {
      subtitlesGroupBox->setEnabled( true );
   }
   else
   {
      subtitlesGroupBox->setEnabled( false );
   }
}

//----- FUNCTIONS
/**
 * FUNCION. Es necesario inicializar los valores de los
 * componentes enla ventana Options, antes de visualizarlos.
 * Esta funcion recibe los parametros del video seleccionado
 * y inicializa estos valores en la ventana de opciones.
 */
void Options_GUI::Initialize( QString fileDir,
                              QString filename,
                              int item_selected,
                              int vbitrate,
                              int abitrate,
                              QString vaspect,
                              bool has_subtitles,
                              QString sub_path,
                              QString sub_font,
                              int sub_scale,
                              int resolution,
                              int owidth,
                              int oheight)
{
   o_width = owidth;
   o_height = oheight;
   comboBox->setCurrentIndex(resolution);
   Select_Resolution(resolution);
   this->fileDir = fileDir;
   filenameEdit->setText( filename );

   videobitrateSpin->setValue( vbitrate );
   audiobitrateSpin->setValue( abitrate );

   if( vaspect == ":autoaspect" )
   {
      aspectComboBox->setCurrentIndex( 0 );
   }
   else
   {
      if ( vaspect == ":aspect=16/9" )
      {
         aspectComboBox->setCurrentIndex( 1 );
      }
      else
      {
         aspectComboBox->setCurrentIndex( 2 );
      }
   }

   if( has_subtitles )
   {
      subtitlesCheckBox->setChecked( true );
      subtitlesGroupBox->setEnabled( true );
      subsEdit->setText( sub_path );
      scaleSpinBox->setValue( sub_scale );
   }
   else
   {
      subtitlesCheckBox->setChecked( false );
      subtitlesGroupBox->setEnabled( false );
      subsEdit->clear();
   }

   m_item_selected = item_selected;

   Check_Subtitles();
}

void Options_GUI::fuentes()
{
   QStringList files,userfiles;
   QDir dir( QCoreApplication::applicationDirPath() );
   QDir::setCurrent( dir.absolutePath() );

#ifdef Q_OS_LINUX
   QDir directory= QDir( font_dir );
   QDir userdir = QDir(QDir::homePath() + "/.QEVEN/Fonts/");
#endif
#ifdef Q_OS_WIN32
   QDir directory = QDir(".\\Fonts\\");
   QSettings ini(QSettings::IniFormat,QSettings::UserScope,
                 QCoreApplication::organizationName(),
                 QCoreApplication::applicationName());
   QDir userdir = QDir(QFileInfo(ini.fileName()).absolutePath()+"/QEVEN/Fonts");
#endif
   files = directory.entryList( QStringList( "*.ttf" ),
                                QDir::Files );
   userfiles = userdir.entryList(QStringList( "*.ttf" ),
                                 QDir::Files );

   cantFUsr = userfiles.count();
   cantFApp = files.count();

   fontComboBox->clear();
   fontComboBox->addItem(tr("----------Usuario---------"));
   fontComboBox->addItems(userfiles);
   fontComboBox->addItem(tr("----------Applicacion-----"));
   fontComboBox->addItems( files );
   int lastP = fontComboBox->findText(lastFont);
   if(lastP == -1 )
      lastP = 0;
   CorrectFont( lastP );
}

void Options_GUI::closeEvent( QCloseEvent */*event*/ )
{
   writeSettings();
}

/**
 * SLOT para agregar subtitulos al video seleccionado
 */
void Options_GUI::Add_Subtitles()
{
   QString videoPath = fileDir;


   QString subtitle = QFileDialog::getOpenFileName( this,
                                                    tr( "Agregar Subtitulos :" ),
                                                    videoPath,
                                                    tr( "Subtitulos (*.srt *.sub)" ) );


#ifdef Q_OS_WIN32
   QStringList path_aux2 = subtitle.split( "/" );
   subtitle= path_aux2.join( "\\" );
#endif
   subsEdit->setText( subtitle );
}

/**
 * Funcion para guardar las configuraciones del sistema
 *
 */
void Options_GUI::writeSettings()
{
#ifdef Q_OS_LINUX
   QSettings settings(  QDir::homePath() + "/.QEVEN/config.ini", QSettings::IniFormat );
#else
   QSettings ini(QSettings::IniFormat,QSettings::UserScope,
                 QCoreApplication::organizationName(),
                 QCoreApplication::applicationName());
   QString dir = QFileInfo(ini.fileName()).absolutePath()+"/QEVEN/config.ini";
   QSettings settings( dir,  QSettings::IniFormat );
#endif
   settings.setValue( "O_Position", pos() );
   settings.setValue( "O_Size", size() );
   settings.setValue( "S_font", fontComboBox->currentText() );
   settings.setValue( "S_scale", scaleSpinBox->value() );
}

/**
 * Funcion para leer las configuraciones del sistema
 */
void Options_GUI::readSettings()
{
#ifdef Q_OS_LINUX
   QSettings settings(  QDir::homePath() + "/.QEVEN/config.ini", QSettings::IniFormat );
#else
   QSettings ini(QSettings::IniFormat,QSettings::UserScope,
                 QCoreApplication::organizationName(),
                 QCoreApplication::applicationName());
   QString dir = QFileInfo(ini.fileName()).absolutePath()+"/QEVEN/config.ini";
   QSettings settings( dir,  QSettings::IniFormat );
#endif
   QPoint pos = settings.value( "O_Position", QPoint( 98, 137 ) ).toPoint();
   QSize size = settings.value( "O_Size", QSize( 417, 419 ) ).toSize();

   move(pos);
   resize(size);

   scaleSpinBox->setValue( settings.value( "S_scale", 3 ).toInt() );
   fontComboBox->setCurrentIndex( fontComboBox->findText(settings.value( "S_font", "FreeSans.ttf" ).toString()) );
}



/**
  * Funcion para importar nuevas fuentes para la aplicacion
  */
void Options_GUI::ImportFont()
{
   QStringList directory = QFileDialog::getOpenFileNames( this,
                                                          tr( "Agregar Archivo :" ),
                                                          QDir::homePath(),
                                                          tr( "Fuentes TrueType (*.ttf)"));
   lastFont=fontComboBox->currentText();

   if ( !directory.isEmpty() )
   {
      for( int i = 0; i < directory.count(); i++ )
      {
         emit importarFont( directory.at( i ) );
      }
      fuentes();
   }
}

/**
  * Funcion que corrgie el combobox de las fuentes
  */
void Options_GUI::CorrectFont(int index)
{
   if(index == 0)
   {
      if(cantFUsr == 0)
         fontComboBox->setCurrentIndex(2);
      else
         fontComboBox->setCurrentIndex(1);
   }
   else
   {
      if(index == cantFUsr + 1)
      {
         fontComboBox->setCurrentIndex(index + 1);
      }
      else
         fontComboBox->setCurrentIndex(index);
   }
}

/**
  * Funcion que corrige el combobox de el bitrate de audio
  */
void Options_GUI::CorrectSB(int value)
{
   audiobitrateSpin->setValue((value/32)*32);
}

/**
   * Funcion que actualiza los edit de las fuentes
   */
void Options_GUI::Select_Resolution(int index)
{
   spwidth->setEnabled(false);
   spheight->setEnabled(false);
   switch(index)
   {
        case 0: spwidth->setValue(o_width);
           spheight->setValue(o_height);
           break;
        case 1: spwidth->setValue(352);
           spheight->setValue(240);
           break;
        case 2: spwidth->setValue(480);
           spheight->setValue(480);
           break;
        case 3: spwidth->setValue(720);
           spheight->setValue(480);
           break;
        case 4: spwidth->setValue(1280);
           spheight->setValue(720);
           break;
        case 5: spwidth->setValue(1920);
           spheight->setValue(1080);
           break;
        case 6: spwidth->setEnabled(true);
           spheight->setEnabled(true);
           break;
        }
}
